import { Request, Response } from 'express';
import { prisma, __path, log } from '../../../index.js';

// Get storageUnit.
function get(req: Request, res: Response) {
	if (req.query.getAll === undefined) {
		// Check if required fields are present.
		if (!req.query.name) {
			res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
			return;
		}
		prisma.storageUnit
			.findUnique({
				where: {
					name: req.query.name.toString()
				},
				// Get category name from relation.
				include: {
					contactInfo: true
				}
			})
			.then((items) => {
				if (items) {
					res.status(200).json(JSON.stringify(items));
				} else {
					res.status(410).json({ error: 'it seems that there is no storageUnit present.' });
				}
			})
			.catch((err) => {
				console.error(err);
				res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
			});
	} else {
		prisma.storageUnit
			.findMany({
				// Get category name from relation.
				include: {
					contactInfo: true
				}
			})
			.then((items) => {
				if (items) {
					res.status(200).json(JSON.stringify(items));
				} else {
					res.status(410).json({ error: 'storageUnit does not exist.' });
				}
			})
			.catch((err) => {
				console.error(err);
				res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
			});
	}
}

// Create storageUnit.
function post(req: Request, res: Response) {
	log.web.debug(JSON.stringify(req.body));
/* 	// Check if required fields are present.
	if (!req.body.name) {
		res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
		return;
	}

	// Save data.
	prisma.storageUnit
		.create({
			data: {
				name: req.body.name
			}
		})
		.then(() => {
			res.status(201).json({ status: 'created' });
		})
		.catch((err) => {
			// TODO Catch if is a duplicate error and show a message to the user
			log.db.error(err);
			res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
		}); */
}

// Update category.
function patch(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id || !req.body.name) {
		res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
		return;
	}

	prisma.storageUnit
		.update({
			where: {
				id: parseInt(req.body.id)
			},
			data: {
				name: req.body.name
			}
		})
		.then(() => {
			res.status(201).json({ status: 'updated' });
		})
		.catch((err) => {
			// TODO Catch if is a duplicate error and show a message to the user
			log.db.error(err);
			res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
		});
}

// Delete category.
async function del(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id) {
		res.status(400).render(__path + '/src/frontend/errors/400.eta.html');
		return;
	}

	// Does the id exist? If not return 410 Gone.
	try {
		const result = await prisma.storageUnit.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(410).json({ error: 'Storage Unit does not exist.' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
	}

	prisma.storageUnit
		.delete({
			where: {
				id: parseInt(req.body.id)
			}
		})
		.then(() => {
			res.status(201).json({ status: 'deleted' });
		})
		.catch((err) => {
			log.db.error(err);
			res.status(500).render(__path + '/src/frontend/errors/dbError.eta.html', { error: err });
		});
}

export default { get, post, patch, del };
